@@ -1,6 +1,5 @@
 # A simple HTTP server implemented using h11 and Trio:
 #   http://trio.readthedocs.io/en/latest/index.html
-# (so requires python 3.5+).
 #
 # All requests get echoed back a JSON document containing information about
 # the request.
@@ -75,17 +74,34 @@
 # - We should probably do something cleverer with buffering responses and
 #   TCP_CORK and suchlike.
 
+import datetime
+import email.utils
 import json
 from itertools import count
-from wsgiref.handlers import format_date_time
 
 import trio
 
 import h11
 
-MAX_RECV = 2 ** 16
+MAX_RECV = 2**16
 TIMEOUT = 10
 
+
+# We are using email.utils.format_datetime to generate the Date header.
+# It may sound weird, but it actually follows the RFC.
+# Please see: https://stackoverflow.com/a/59416334/14723771
+#
+# See also:
+# [1] https://www.rfc-editor.org/rfc/rfc9110#section-5.6.7
+# [2] https://www.rfc-editor.org/rfc/rfc7231#section-7.1.1.1
+# [3] https://www.rfc-editor.org/rfc/rfc5322#section-3.3
+def format_date_time(dt=None):
+    """Generate a RFC 7231 / RFC 9110 IMF-fixdate string"""
+    if dt is None:
+        dt = datetime.datetime.now(datetime.timezone.utc)
+    return email.utils.format_datetime(dt, usegmt=True)
+
+
 ################################################################
 # I/O adapter: h11 <-> trio
 ################################################################
@@ -114,7 +130,13 @@ class TrioHTTPWrapper:
         # appropriate when 'data' is None.
         assert type(event) is not h11.ConnectionClosed
         data = self.conn.send(event)
-        await self.stream.send_all(data)
+        try:
+            await self.stream.send_all(data)
+        except BaseException:
+            # If send_all raises an exception (especially trio.Cancelled),
+            # we have no choice but to give it up.
+            self.conn.send_failed()
+            raise
 
     async def _read_from_peer(self):
         if self.conn.they_are_waiting_for_100_continue:
@@ -177,7 +199,7 @@ class TrioHTTPWrapper:
         # HTTP requires these headers in all responses (client would do
         # something different here)
         return [
-            ("Date", format_date_time(None).encode("ascii")),
+            ("Date", format_date_time().encode("ascii")),
             ("Server", self.ident),
         ]
 
